//+------------------------------------------------------------------+
//|                                         Championship_2010_v2.mq4 |
//|                                                        avoitenko |
//|                        https://login.mql5.com/ru/users/avoitenko |
//+------------------------------------------------------------------+
#property copyright ""
#property link      "https://login.mql5.com/ru/users/avoitenko"
#property link      "http://fxmts.net/files/"
#property link      "2.00"

//+------------------------------------------------------------------+
//| Input parameters                                                 |
//+------------------------------------------------------------------+
extern string  General         = "=== General options ===";
extern int     PERIOD          =     250; // Calculation period
extern int     STOP_LOSS       =      50; // Level of stop loss
extern int     TAKE_PROFIT     =     100; // Level of take profit
extern int     INSIDE_LEVEL    =      30; // Level back into the channel
extern int     TRAILING_STOP   =      20; // Trailing stop level
extern int     TRAILING_STEP   =      10; // Trailing step level
extern int     ORDER_STEP      =      10; // Step movement of a pending order
extern int     SLIPPAGE        =       2; // Slippage
extern double  LOT             =       5; // Lots
extern bool    LOT_IS_FIXED    =    true; // Lot type
extern bool    LOT_CORRECTION  =   false; // Correction of lot size
extern int     MAGIC_NUMBER    =     867; // Unique number Adviser
extern string  Additional      = "=== Additional options ===";
extern bool    PRINT_REPORT    =   false; // Log output

//+------------------------------------------------------------------+
//| Global variables                                                 |
//+------------------------------------------------------------------+
int stop_loss;            // stop loss
int take_profit;          // take profit
int inside_level;         // level back into the channel
int trailing_level;       // trailing stop
int trailing_step;        // trailing step
int order_step;           // order step
int slippage;             // slippage
double lot;               // lots
//---
bool buy_open    = false; // buy  flag
bool sell_open   = false; // sell flag
double CalcHigh;          // resistance level
double CalcLow;           // support level
//---
double order_open_price;  // order open price
double spread;            // spread
int stop_level;           // minimum level of the price to install a Stop Loss / Take Profit
int order_type;           // order type
int order_ticket;         // order ticket 
int orders_total;
double order_stop_loss;
double sl;
double tp;
double target;
double price;
string err_msg;
bool start;
bool alert;
double margin;

//+------------------------------------------------------------------+
//| initialization function of the expert                            |
//+------------------------------------------------------------------+
int init()
//+------------------------------------------------------------------+
{
   
   start = true;
   alert = false;
//--- Checking the calculation period
   if(PERIOD < 3)
   {
      err_msg = "Error, invalid parameter PERIOD = " + DoubleToStr(PERIOD,0) + " allowable values >= 3";
      start = false;
   }

//--- Checking the stop loss value
   if(STOP_LOSS <= 0) 
   {
      err_msg = "Error, invalid parameter STOP_LOSS = " + DoubleToStr(STOP_LOSS,0) + " allowable values > 0";
      start = false;
   }

//--- Checking the Take Profit value
   if(TAKE_PROFIT<0)
   {
      err_msg = "Error, invalid parameter TAKE_PROFIT = " + DoubleToStr(TAKE_PROFIT,0) + " allowable values >= 0";
      start = false;
   }
//--- Checking the level back into the channel
   if(INSIDE_LEVEL<1)
   {
      err_msg = "Error, invalid parameter INSIDE_LEVEL = " + DoubleToStr(INSIDE_LEVEL,0) + " allowable values >= 1";
      start = false;
   }

//--- Checking the trailing stop value
   if(TRAILING_STOP<0)
   {
      err_msg = "Error, invalid parameter TRAILING_STOP = " + DoubleToStr(TRAILING_STOP,0) + " allowable values >= 0";
      start = false;
   }
//--- Checking step move the trailing stop   
   if(TRAILING_STEP < 0)
   {
      err_msg = "Error, invalid parameter TRAILING_STEP = " + DoubleToStr(TRAILING_STEP,0) + " allowable values >= 0";
      start = false;
   }
//---   
   if(TRAILING_STEP > TRAILING_STOP)
   {
      err_msg = "Error, invalid parameter TRAILING_STEP = " + DoubleToStr(TRAILING_STEP,0) + " allowable values < TRAILING_STOP";
      start = false;
   }
//--- Checking the step movement of a pending order
   if(ORDER_STEP<0)
   {
      err_msg = "Error, invalid parameter ORDER_STEP = " + DoubleToStr(ORDER_STEP,0) + " allowable values >=0";
      start = false;
   }
//--- Checking slippage
   if(SLIPPAGE<0)
   {
      err_msg = "Error, invalid parameter SLIPPAGE = " + DoubleToStr(SLIPPAGE,0) + " allowable values >=0";
      start = false;
   }
//--- 
   if(!OrderCalcMargin(Symbol(),1,Ask,margin) && !LOT_IS_FIXED)
   {
      err_msg = "Error, For this symbol" + Symbol() +" can only trade in fixed lot: LOT_IS_FIXED = true";
      start = false;
   }

//---
   stop_loss      = STOP_LOSS;
   take_profit    = TAKE_PROFIT;
   inside_level   = INSIDE_LEVEL;
   trailing_level = TRAILING_STOP;
   trailing_step  = TRAILING_STEP;
   slippage       = SLIPPAGE;
   order_step     = ORDER_STEP;

//--- If the price includes 3 / 5 decimal places
   if(Digits == 3 || Digits == 5)
   {
      stop_loss      = stop_loss      * 10;
      take_profit    = take_profit    * 10;
      inside_level   = inside_level   * 10;
      trailing_level = trailing_level * 10;
      trailing_step  = trailing_step  * 10;
      slippage       = slippage       * 10;
      order_step     = order_step     * 10;
   }
   start();
   return(0);
}

//+------------------------------------------------------------------+
//|   the main function of the expert                                |
//+------------------------------------------------------------------+
int start()
//+------------------------------------------------------------------+
  {
   
   if(!IsExpertEnabled() && !IsTesting())
   {
      err_msg = "Warning: Job advisor is not allowed. Please click \"Expert Advisors\"";
      start=false;
   } 

//--- If there is an error in the values of external variables, then - exit
   if(!start)
   {
      if(!alert)
      {
         Alert("Error input variables dviser","\n",err_msg);
         Print(err_msg);
         alert=true;
      }
      return(-1);
   }

//--- Check availability of historical data for M5 timeframe
   if(iBars(Symbol(), PERIOD_M5) < PERIOD)
   {
      Print("Little historical data for trade");
      return;
   }
//--- Retrieving array data High and Low
   else
   {
//--- Finding the minimum and maximum
      CalcHigh = 0.0;
      CalcLow  = 0.0;
      
      int high_index = iHighest(Symbol(), PERIOD_M5, MODE_HIGH, PERIOD,0);
      int low_index = iLowest(Symbol(), PERIOD_M5, MODE_LOW, PERIOD,0);
      
      CalcHigh = iHigh(Symbol(), PERIOD_M5, high_index);
      CalcLow  =  iLow(Symbol(), PERIOD_M5, low_index);
   }

//--- Exit if CalcHigh and CalcLow not counted
   if(CalcHigh < Point || CalcLow < Point) return;

//--- Upgrade options
   stop_level = MarketInfo(Symbol(),MODE_STOPLEVEL);
   spread = NormalizeDouble(Ask - Bid, Digits); 

//--- The conditions of return inside the channel
   if(Ask <= NormalizeDouble(CalcHigh - inside_level * Point, Digits)) buy_open  = true;
   if(Bid >= NormalizeDouble(CalcLow  + inside_level * Point, Digits)) sell_open = true;

//--- Processing of open orders
   WorkWithPositions();

//--- Processing of pending orders
   WorkWithPendidngOrders();

//--- Set a new pending order to buy
   if(buy_open) OpenOrderBuyStop();

//--- Set a new pending order to sell
   if(sell_open) OpenOrderSellStop();

   return(0);
 }
 

//+------------------------------------------------------------------+
//|   Working with the positions                                     |
//+------------------------------------------------------------------+
void WorkWithPositions()
//+------------------------------------------------------------------+
{
   orders_total = OrdersTotal();
   for(int pos = orders_total-1; pos>=0; pos--)
   {
      if(OrderSelect(pos, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() != Symbol() || OrderMagicNumber() != MAGIC_NUMBER) continue; //not my order
         {
            order_open_price = OrderOpenPrice();
            order_type       = OrderType();
            order_stop_loss  = OrderStopLoss();
            order_ticket     = OrderTicket();
            
            if(order_type == OP_BUY)
            {
               buy_open = false;

               //--- Trailing stop
               if(order_stop_loss == 0 || trailing_level == 0 || trailing_step == 0) continue; // condition under which the trailing stop is not working
   
               target = NormalizeDouble(MathMax(order_open_price, order_stop_loss) + trailing_level*Point, Digits);
 
               if(Bid > target)
               {
                  if(order_stop_loss < order_open_price)
                  {
                     if(OrderModify(order_ticket, order_open_price, order_open_price, OrderTakeProfit(), 0, Blue))
                        if(PRINT_REPORT)Print("Moving stop loss order Buy #", order_ticket);  
                     else
                        if(PRINT_REPORT)Print("Error moving stop loss order Buy #", order_ticket);
                  }                     
                  else
                  {
                     sl=NormalizeDouble(order_stop_loss + trailing_step*Point,Digits);
                     
                     if(OrderModify(order_ticket, order_open_price, sl, OrderTakeProfit(), 0, Blue))
                        if(PRINT_REPORT)Print("Moving stop loss order Buy #", order_ticket);  
                     else
                        if(PRINT_REPORT)Print("Error moving stop loss order Buy #", order_ticket);
                  }   
                  
                  Sleep(2000);
                  RefreshRates(); 

               }
               continue;
            }// end OP_BUY

            else if(order_type == OP_SELL)
            {
               sell_open = false;

               //--- Trailing Stop
               if(order_stop_loss == 0 || trailing_level == 0 || trailing_step == 0) continue;

               target = NormalizeDouble(MathMin(order_open_price, order_stop_loss) - trailing_level*Point, Digits);

               if(Ask < target)
               {
   
                  if(order_stop_loss > order_open_price)
                  {
                     if(OrderModify(order_ticket, order_open_price, order_open_price, OrderTakeProfit(),0,Blue))
                        if(PRINT_REPORT)Print("Moving stop loss order Sell #",order_ticket);
                     else
                        if(PRINT_REPORT)Print("Error moving stop loss order Sell #",order_ticket);
                  }
                  else
                  {
                     sl = NormalizeDouble(order_stop_loss - trailing_step * Point,Digits);               

                     if(OrderModify(order_ticket, order_open_price, sl, OrderTakeProfit(),0,Blue))
                        if(PRINT_REPORT)Print("Moving stop loss order Sell #",order_ticket);
                     else
                        if(PRINT_REPORT)Print("Error moving stop loss order Sell #",order_ticket);
                  }
                     
                  Sleep(2000);
                  RefreshRates(); 
               }

            }// end OP_SELL
            
         }// end if

      }// end if select                

   }// end for
}

//+------------------------------------------------------------------+
//|   Working with pending orders                                    |
//+------------------------------------------------------------------+
void WorkWithPendidngOrders()
//+------------------------------------------------------------------+
{
   //--- Processing pending orders
   for(int pos=0; pos < OrdersTotal(); pos++)
   {
      if(OrderSelect(pos,SELECT_BY_POS,MODE_TRADES))
      {

         if( OrderSymbol() != Symbol() || OrderMagicNumber() != MAGIC_NUMBER) continue;//not my order
         
         order_ticket = OrderTicket();
         order_type = OrderType();
         order_open_price = OrderOpenPrice();
         
         if(order_type == OP_BUYSTOP)
         {
            buy_open = false;
            
            //--- Conditions for the movement of a pending order
            if(( NormalizeDouble(CalcHigh - order_step * Point + spread, Digits) < NormalizeDouble(order_open_price - order_step * Point + spread, Digits)) && 
               ( NormalizeDouble(CalcHigh - order_step * Point + spread, Digits) > Ask + stop_level * Point))
            {

               price = NormalizeDouble(CalcHigh - order_step * Point + spread, Digits);

               if(stop_loss == 0) sl = 0;
               else sl = NormalizeDouble(CalcHigh - order_step * Point - MathMax(stop_loss, stop_level) * Point, Digits);

               if(take_profit == 0) tp = 0;
               else tp = NormalizeDouble(CalcHigh - order_step * Point + MathMax(take_profit, stop_level) * Point, Digits);
               
               if (OrderModify(order_ticket, price, sl, tp, 0))
                  if(PRINT_REPORT)Print("Modified order BuyStop # ",order_ticket);
               else
                  if(PRINT_REPORT)Print("Error modified order BuyStop # ",order_ticket);

               Sleep(2000);
               RefreshRates(); 
            }
               
         }//end ORDER_TYPE_BUY_STOP  

         else if(order_type == OP_SELLSTOP)
           {
            sell_open = false;
            
            //--- Conditions for the movement of a pending order
            if(( NormalizeDouble(CalcLow + order_step * Point, Digits) > order_open_price + order_step * Point)&& 
               ( NormalizeDouble(CalcLow + order_step * Point, Digits) < Bid - stop_level * Point))
              {
               price = NormalizeDouble(CalcLow + order_step * Point, Digits);

               if(stop_loss == 0) sl = 0;
               else sl = NormalizeDouble(price + MathMax(stop_loss * Point, stop_level * Point) + spread, Digits);

               if(take_profit == 0) tp = 0;
               else tp = NormalizeDouble(price - MathMax(take_profit * Point, stop_level * Point) + spread, Digits);

               if (OrderModify(OrderTicket(), price, sl, tp, 0))
                  if(PRINT_REPORT)Print("Modified order SellStop # ",order_ticket);
               else
                  if(PRINT_REPORT)Print("Error modified order SellStop # ",GetLastError());
              
               Sleep(2000);
               RefreshRates(); 

              }

           }//end ORDER_TYPE_SELL_STOP   

        }// end order_ticket

     }// end for
  }

//+------------------------------------------------------------------+
//|   Set the pending order to buy                                   |
//+------------------------------------------------------------------+
void OpenOrderBuyStop()
//+------------------------------------------------------------------+
{
   lot = Calculate_Lot(LOT, LOT_IS_FIXED, OP_BUY); //Do not specify ORDER_TYPE_BUY_STOP. This constant is not processed in OrderCalcMargin!

   price = NormalizeDouble(CalcHigh - order_step * Point, Digits);

   //--- adjust the price of orders
   if(NormalizeDouble(Ask + stop_level * Point, Digits) > price) return(0);//price = NormalizeDouble(Ask + stop_level * Point, Digits);
  
   //--- Calculation of stop loss
   if(stop_loss == 0) sl = 0;
   else sl = NormalizeDouble(price - MathMax(stop_loss, stop_level) * Point - spread, Digits);
   
   //--- Calculation of take profit
   if(take_profit == 0) tp = 0;
   else tp = NormalizeDouble(price + MathMax(take_profit, stop_level) * Point - spread, Digits);

   if(OrderSend(Symbol(), OP_BUYSTOP, lot, price, 0, sl, tp, DoubleToStr(MAGIC_NUMBER,0), MAGIC_NUMBER, 0, Blue) >= 0)
   {
      buy_open = false;
      if(PRINT_REPORT)Print("Opened an order BuyStop #" + DoubleToStr(OrderTicket(),0));
   }
   else 
   {
      if(PRINT_REPORT)Print("Error opened an order BuyStop, error code:" + GetLastError());
   }
   
   Sleep(2000);
   RefreshRates();

}
//+------------------------------------------------------------------+  
//|   Set the pending order to sell                                  |
//+------------------------------------------------------------------+
void OpenOrderSellStop()
//+------------------------------------------------------------------+
  {
   lot = Calculate_Lot(LOT, LOT_IS_FIXED, OP_SELL); //Do not specify ORDER_TYPE_SELL_STOP. This constant is not processed in OrderCalcMargin!

   price = NormalizeDouble(CalcLow + order_step * Point, Digits);
   
   //--- adjust the price of orders
   if(NormalizeDouble(Bid - stop_level * Point, Digits) < price) return(0);//price = NormalizeDouble(Bid - stop_level * Point, Digits);
   
   //--- The calculation of stop loss
   if(stop_loss == 0) sl = 0;
   else sl = NormalizeDouble(price + MathMax(stop_loss, stop_level)* Point + spread, Digits);
   
   //--- The calculation of take profit
   if(take_profit == 0) tp = 0;
   else tp = NormalizeDouble(price - MathMax(take_profit, stop_level) * Point + spread, Digits);

   if(OrderSend( Symbol(), OP_SELLSTOP, lot, price, 0, sl, tp, DoubleToStr(MAGIC_NUMBER,0),MAGIC_NUMBER, 0, Blue)>=0)
   {
      sell_open = false;
      if(PRINT_REPORT)Print("Opened an order SellStop #" + DoubleToStr(OrderTicket(),0));
   }
   else 
   {
      if(PRINT_REPORT)Print("Error opened an order SellStop, error code:" + GetLastError());
   }
   
   Sleep(2000);
   RefreshRates();
}

//+------------------------------------------------------------------+
//|   Calculation of the lot                                         |
//+------------------------------------------------------------------+
double Calculate_Lot(double lot_value, bool lot_is_fixed, int direction)
//+------------------------------------------------------------------+
  {
   double acc_free_margin = AccountFreeMargin();
   double calc_margin;
   double price = 0;

   if(direction == OP_BUY)  price = Ask;
   if(direction == OP_SELL) price = Bid;

   if(lot_is_fixed)
   {
      if(LOT_CORRECTION)//--- Correction of lot size
      {
         OrderCalcMargin(Symbol(), lot_value, price, calc_margin);
         
         //--- Lot size correction of up to 90% of free margin
         if(acc_free_margin < calc_margin)
         {
            lot_value = lot_value * acc_free_margin * 0.9 / calc_margin;
            if(PRINT_REPORT)Print("Adjusted value of the lot: ",lot);
         }
      }

   }
   else
   {
         //--- value of free margin for open position
         OrderCalcMargin(Symbol(), 1, price, calc_margin);
         lot_value = acc_free_margin * 0.01 * LOT / calc_margin;
   }

   return(NormalizeLot(lot_value));
  }

//+------------------------------------------------------------------+
//|   Normalization of the lot size                                  |
//+------------------------------------------------------------------+
double NormalizeLot(double lot_value)
//+------------------------------------------------------------------+
{
   double lot_min  = MarketInfo(Symbol(), MODE_MINLOT);
   double lot_max  = MarketInfo(Symbol(), MODE_MAXLOT);
   double lot_step = MarketInfo(Symbol(), MODE_LOTSTEP);

   if(lot_value <= lot_min ) lot_value = lot_min;                 // checking for minimum lot
   else  if(lot_value >= lot_max ) lot_value = lot_max;           // checking the maximum lot
   else lot_value  = MathFloor(lot_value / lot_step) * lot_step;  // rounding to the nearest

   return (NormalizeDouble(lot_value,2)); // normalization
}

//+------------------------------------------------------------------+  
//|   Calculation of the free margin                                 |
//|   Applies only for standard Forex symbol                         |
//+------------------------------------------------------------------+  
bool  OrderCalcMargin(string    symbol,      // Symbol name
                      double    volume,      // Lots
                      double    price,       // Open price
                      double&   margin)      // output value
//+------------------------------------------------------------------+
{
   string first    = StringSubstr(symbol,0,3);
   string second   = StringSubstr(symbol,3,3);
   string currency = AccountCurrency();        
   double leverage = AccountLeverage();        

   double contract = MarketInfo(symbol, MODE_LOTSIZE);
   margin = 0.0;

//---   
   if(StringLen(symbol) != 6)
     {
      Print("OrderCalcMargin: ",symbol," - must be a standard forex symbol by type: XXXYYY");
      return(false);
     }
//--- 
   if( price <= 0.0 || contract <= 0) 
     {
      Print("OrderCalcMargin: No market information for ",symbol);
      return(false);
     }
//---
   if(first == currency)
   {  
      margin=contract * volume / leverage; // USDxxx
      return(true);
   }
//---   
   if((second == currency))
   {
      margin=contract * price * volume / leverage; // xxxUSD
      return(true);
   }
//---
   Print("OrderCalcMargin: impossible to calculate for ",symbol);
   return(false);
}



